var preprocess = require('./shark-preprocess');
var sharkWebpack = require('./shark-webpack');
var baseprocess = require('./shark-baseprocess');
var browersync = require('./shark-browersync');
var ftl2html = require('./shark-ftl2html');
var sharkAjax = require('./shark-ajax');
var sharkClean = require('./shark-clean');
var sharkCopy = require('./shark-copy');
var sharkJava = require('./shark-java');
var sharkZip = require('./shark-zip');
var express = require('express');
var deploy = require('./shark-deploy');
var path = require('path');

var automation = {};

/**
 * 注册各个task
 * conf
 *    baseConf: 对应的shark-deploy-conf.json的配置信息
 *    webpack:   对应的webpack的配置信息
 *    ...  各个插件的配置，允许修改task中使用的插件的默认配置
 */
automation.registerTasks = function(conf) {
    preprocess(conf);
    sharkWebpack.registerServerTask(conf);
    sharkClean(conf);
    baseprocess(conf);
    sharkJava(conf);
    sharkCopy(conf);
    deploy(conf);
    sharkZip(conf);
};

automation.registerServer = function(conf) {
    var baseConf = conf.baseConf;
    var prePath = path.join(baseConf.rootPath, baseConf.tmpDir, 'step1');
    var webPath = path.join(baseConf.rootPath, baseConf.webapp);
    var htmlPath = path.join(webPath, baseConf.htmlPath);
    var router = express.Router();
    router.use(sharkWebpack.serveStatic(conf));
    router.use(express.static(prePath));
    if (baseConf.projectPath) {
        router.use(baseConf.projectPath, express.static(htmlPath));
    }
    router.use(express.static(htmlPath));
    router.use(express.static(webPath));
    router.use('/', function(req, res, next) {
        ftl2html(req, res, next, baseConf);
    });
    router.use('/', sharkAjax.localRequset(baseConf));

    preprocess(conf, 'serve');

    browersync(conf);

    return router;
};

module.exports = automation;