var path = require('path');
var JaySchema = require('jayschema');
var fs = require('fs');
var request = require('request');

function validateRequest(config, req) {
    var requestRoot = path.join(config.rootPath, 'src/test', 'validator/request');
    var reqPath = req.path;
    var schemaFile = path.join(requestRoot, reqPath);
    if (fs.existsSync(schemaFile)) {
        var schema = JSON.parse(fs.readFileSync(schemaFile));
        var data = Object.assign({}, req.body, req.query);

        var JSONValidator = new JaySchema();
        JSONValidator.validate(data, schema, function(errs) {
            if (errs) {
                // error
                console.error(errs);
                return { code: "415", errorCode: "前端Request参数格式校验发现Error，请查看控制台" };
            } else {
                return true;
            }
        });
    } else {
        return true;
    }
}

function validateResponse(config, req, resp) {
    // rsp - original response from the target
    var data = JSON.parse(resp);
    // validate reponse schema
    var responseRoot = path.join(config.rootPath, 'src/test', 'validator/response');
    var reqPath = req.path;

    var schemaFile = path.join(responseRoot, reqPath);
    if (fs.existsSync(schemaFile)) {
        var schema = JSON.parse(fs.readFileSync(schemaFile));

        var JSONValidator = new JaySchema();
        JSONValidator.validate(data, schema, function(errs) {
            if (errs) {
                // error
                console.error(errs);
                return { code: "416", errorCode: "服务器Response格式校验发现Error，请查看控制台" };
            } else {
                return true;
            }
        });
    } else {
        return true;
    }
}

var sharkAjax = {};

sharkAjax.localRequset = function(config) {
    console.log("localRequset");
    return function(req, res, next) {
        var mockPath = path.join(config.rootPath, config.mock);
        var reqPath = req.path;
        var mockFile = path.join(mockPath, reqPath);
        if (fs.existsSync(mockFile)) {
            var validate = validateRequest(config, req);
            if (validate !== true) {
                res.send(validate);
            } else {
                res.send(fs.readFileSync(mockFile, 'utf-8'));
            }
        } else {
            next();
        }
    };
};

sharkAjax.remoteRequest = function(config) {
    return function(req, res, next) {
        var originalUrl = req.originalUrl;
        var reqPath = req.path;
        var remoteUrl = path.join(config.remoteUrl, reqPath);
        var r = request({
            url: remoteUrl,
            jar: true
        });
        req.pipe(r);
        r.on('error', function(err) {
            console.log(err);
            res.send(err);
        }).on('response', function(response) {
            var rst = '';
            response.setEncoding('utf-8');
            response.on('data', function(chunk) {
                rst += chunk;
            });
            response.on('end', function(chunk) {
                rst += chunk;
                var validate = validateResponse(config, req, rst);
                if (validate !== true) {
                    res.send(validate);
                } else {
                    r.pipe(res);
                }
            });
        });
    };
};

module.exports = sharkAjax;