var gulp = require('gulp'),
    useref = require('gulp-useref'),
    uglify = require('gulp-uglify'),
    minifyCss = require('gulp-minify-css'),
    imagemin = require('gulp-imagemin'),
    pngquant = require('imagemin-pngquant'),
    rev = require('gulp-rev'),
    argv = require('yargs').argv,
    revReplace = require('gulp-rev-replace'),
    ngAnnotate = require('gulp-ng-annotate'),
    path = require('path'),
    os = require('os'),
    autoprefixer = require('gulp-autoprefixer'),
    base64 = require('gulp-base64'),
    clean = require('../shark-clean'),
    extend = require('extend'),
    runSequence = require('run-sequence'),
    gulpif = require('gulp-if');


module.exports = function(appConfig) {
    var baseConf = appConfig.baseConf;
    var ifWebpack = baseConf.ifwebpack;
    var target = argv.target;
    /***------------init the config opts start---------------***/
    var webappDir = path.join(baseConf.rootPath, baseConf.webapp);
    var defaultautoprefixerConf = {
        remove: false,
        browsers: [
            "last 2 versions"
        ]
    };
    var autoprefixerConfig = extend({}, defaultautoprefixerConf, appConfig.autoprefixer);
    var ifFtl = !!(baseConf.ftl2html) ? true : false;
    var tmp1 = path.join(baseConf.rootPath, baseConf.tmpDir, 'step1');
    var tmp2 = path.join(baseConf.rootPath, baseConf.tmpDir, 'step2');
    var tmp3 = path.join(baseConf.rootPath, baseConf.tmpDir, 'step3');
    /***------------init the config opts end ---------------***/

    /***------------the condition for uglifyjs ---------------***/
    var uglifyJs = function(file) {
        if (/\-min.js$/.test(file.path)) {
            return false;
        } else if (/.js$/.test(file.path)) {
            return true;
        }
        return false;
    }

    /***------------the condition for ngAnnotate ---------------***/
    var ifAngular = function(file) {
        if (/.js$/.test(file.path) && appConfig.projectType === "Angular") {
            return true;
        }
        return false;
    }

    //init base64Conf 
    var defaultBase64Conf = {
        extensions: ['png'],
        maxImageSize: 6 * 1024,
        debug: false
    };
    var base64Conf = extend({}, defaultBase64Conf, appConfig.base64Conf);
    /***-------------ftl (js css) concat and minimize  start ---------------***/
    gulp.task('useref-ftl', function() {
        return gulp.src(path.join(webappDir, '**/*.ftl'))
            .pipe(useref({
                searchPath: tmp1
            }))
            .pipe(gulpif(ifAngular, ngAnnotate()))
            .pipe(gulpif(uglifyJs, uglify()))
            .pipe(gulpif('*.css', base64(base64Conf)))
            .pipe(gulpif('*.css', autoprefixer(autoprefixerConfig)))
            .pipe(gulpif('*.css', minifyCss()))
            .pipe(gulp.dest(tmp2));
    });
    /***-------------ftl (js css) concat and minimize end ---------------***/

    /***-------------js copy start  ---------------***/
    gulp.task('copy-js', function() {
        return gulp.src(path.join(tmp1, '**/*.js'))
            .pipe(gulp.dest(tmp2));
    });
    if (ifWebpack) {
        gulp.task('useref', ['useref-html', 'useref-ftl', 'copy-js']);
    } else {
        gulp.task('useref', ['useref-html', 'useref-ftl']);
    }
    /***-------------js copy end ---------------***/

    /***-------------html (js css) concat and minimize start ---------------***/
    gulp.task('useref-html', function() {
        return gulp.src(path.join(webappDir, '**/*.{html,htm}'))
            .pipe(useref({
                searchPath: tmp1
            }))
            .pipe(gulpif(ifAngular, ngAnnotate()))
            .pipe(gulpif(uglifyJs, uglify()))
            .pipe(gulpif('*.css', base64(base64Conf)))
            .pipe(gulpif('*.css', autoprefixer(autoprefixerConfig)))
            .pipe(gulpif('*.css', minifyCss()))
            .pipe(gulp.dest(tmp2));
    });
    /***-------------html (js css) concat minimize end ---------------***/

    /***----------------------- imagemin start ---------------------***/
    var imgminType = appConfig.imgminType || ["jpg", "jpeg", "gif", "png"];
    var imgType = imgminType.join(",");
    gulp.task('imagemin', function() {
        return gulp.src(path.join(webappDir, "**/*.{" + imgType + "}"))
            .pipe(imagemin({
                // jpg
                progressive: true,
                // for png
                use: [pngquant({
                    quality: 90
                })]
            }))
            .pipe(gulp.dest(tmp2));
    });
    /***---------------------- imagemin end ------------------***/

    /***-------------------- image revision start ------------------***/
    gulp.task("revision-image", function() {
        return gulp.src(path.join(webappDir, "**/*.{" + imgType + "}"))
            .pipe(rev())
            .pipe(gulp.dest(tmp3))
            .pipe(rev.manifest('image-rev-manifest.json'))
            .pipe(gulp.dest(tmp3))
    });
    /***------------------ image revision end ------------------***/

    /***----------------js css revision start ---------------***/
    gulp.task("revision-css", function() {
        return gulp.src([path.join(tmp2, "**/*.css")])
            .pipe(rev())
            .pipe(gulp.dest(tmp3))
            .pipe(rev.manifest('style-rev-manifest.json'))
            .pipe(gulp.dest(tmp3))
    });

    gulp.task("revision-js", function() {
        return gulp.src([path.join(tmp2, "**/*.js")])
            .pipe(rev())
            .pipe(gulp.dest(tmp3))
            .pipe(rev.manifest('js-rev-manifest.json'))
            .pipe(gulp.dest(tmp3))
    });
    /***-------------js css revision end ---------------***/

    function getMimgUrlPrefix() {
        var prefix = baseConf.mimgURLPrefix[target];
        if (typeof prefix === 'string') {
            var mimgUrl = prefix + '/' + getPath(path.join(baseConf.product, baseConf.staticVersion));
            return mimgUrl;
        }
        throw new Error('build target [' + target + '] undefined in mimgURLPrefix');
    }

    /**
     * 兼容window与mac下的路径问题
     *
     * @param  {string} rPath 路径
     * @return {string}       处理后的路径
     */
    function getPath(rPath) {
        if (os.platform() === 'win32') {
            return (rPath || '').replace(/\\/ig, '/');
        } else {
            return rPath || '.';
        }
    }

    /***------------- revreplace-css start ---------------***/

    gulp.task("revreplace-css", function() {
        var manifest = gulp.src([
            path.join(tmp3, '/image-rev-manifest.json')
        ]);

        return gulp.src(path.join(tmp2, "**/*.css"))
            .pipe(revReplace({
                manifest: manifest,
                replaceInExtensions: ['.css'],
                prefix: getMimgUrlPrefix()
            }))
            .pipe(gulp.dest(tmp2));
    });
    /***------------- revreplace-css end ---------------***/

    /***------------- revreplace-js start ---------------***/
    gulp.task("revreplace-js", function() {
        var manifest = gulp.src([
            path.join(tmp3, '/image-rev-manifest.json')
        ]);

        return gulp.src(path.join(tmp2, "**/*.js"))
            .pipe(revReplace({
                manifest: manifest,
                replaceInExtensions: ['.js'],
                prefix: getMimgUrlPrefix()
            }))
            .pipe(gulp.dest(tmp2));
    });
    /***------------- revreplace-js end ---------------***/

    /***------------- revreplace start ---------------***/
    gulp.task("revreplace-ftl", function() {
        var manifest = gulp.src([
            path.join(tmp3, 'style-rev-manifest.json'),
            path.join(tmp3, '/js-rev-manifest.json'),
            path.join(tmp3, '/image-rev-manifest.json')
        ]);

        return gulp.src(path.join(tmp2, "**/*.ftl"))
            .pipe(revReplace({
                manifest: manifest,
                replaceInExtensions: ['.ftl'],
                prefix: getMimgUrlPrefix()
            }))
            .pipe(gulp.dest(tmp3));
    });

    gulp.task("revreplace-html", function() {
        var manifest = gulp.src([
            path.join(tmp3, '/style-rev-manifest.json'),
            path.join(tmp3, '/js-rev-manifest.json'),
            path.join(tmp3, '/image-rev-manifest.json')
        ]);
        return gulp.src(path.join(tmp2, "**/*.{html,htm}"))
            .pipe(revReplace({
                manifest: manifest,
                replaceInExtensions: ['.html'],
                prefix: getMimgUrlPrefix()
            }))
            .pipe(gulp.dest(tmp3));
    });
    /***------------- revreplace end ---------------***/
}