var deployGit = require('shark-deploy-git'),
    path = require('path'),
    argv = require('yargs').argv,
    gulp = require('gulp');

function getRepo(baseConf, target, isStatics) {
    var repo = 'http://' + baseConf.deploy.account + ':' + baseConf.deploy.password + '@git.internal/';
    if (target === 'online') {
        repo += baseConf.onlineGitDir;
    } else if (target === 'test') {
        repo += baseConf.testGitDir;
    }
    repo = repo + '/' + baseConf.product;
    if (isStatics) {
        repo += '-statics';
    }
    repo += '.git';
    return repo;
};
var deploy = function(conf) {
    // use deployConfigFile to refer to another config file
    var baseConf = conf.baseConf;
    // build dir
    var buildDir = baseConf.build;
    var buildWebappDir = path.join(buildDir, baseConf.buildWebapp);
    var buildStaticDir = path.join(buildDir, baseConf.buildStatic);
    if (typeof baseConf.deployConfigFile === 'string') {
        config = require(baseConf.deployConfigFile);
    }
    gulp.task('deploy', function(cb) {
        var branch = argv.branch;
        var target = argv.target;
        if (!branch) {
            throw new Error('--branch should be provided. ex: gulp deploy --branch master');
        }
        if (!target) {
            throw new Error('--target should be provided. ex: gulp deploy --target test');
        }
        if (target !== 'online' && target !== 'test') {
            throw new Error('--target should be online or test');
        }

        // webapp deploy
        var repo = getRepo(baseConf, target, false);
        var options = {
            branch: branch,
            build: buildWebappDir,
            deploy: path.join(baseConf.deploy.webapp, target),
            repo: repo
        };
        // console.log(options);
        deployGit(options);

        // statics deploy
        var repo = getRepo(baseConf, target, true);
        var staticOptions = {
            branch: branch,
            build: buildStaticDir,
            deploy: path.join(baseConf.deploy.static, target, baseConf.staticField, baseConf.product),
            repo: repo
        };
        // console.log(staticOptions);
        deployGit(staticOptions);
    });
};
module.exports = deploy;