var path = require('path');
var fs = require('fs');
var gulp = require('gulp');
var originalWebpack = require('webpack');
var gulpWebpack = require('gulp-webpack');
var webpackConf = require('./webpack.config.js');
var dllConf = require('./webpack.dll.config.js');
var webpackDev = require('webpack-dev-middleware');
var webpackHot = require('webpack-hot-middleware');
var express = require('express');
var extend = require('extend');

var sharkWebpack = {};

sharkWebpack.registerServerTask = function(conf) {
    var baseConf = addCustomConf(webpackConf('server', conf.baseConf), conf.webpack);
    if (conf.dllPlugin) {
        generateDlllib('server', conf);
        //baseConf.plugins.push(new originalWebpack.DllReferencePlugin(getDllReferencePlugin(conf)));
    }
    var dest = path.join(conf.baseConf.rootPath, conf.baseConf.tmpDir, 'step1');
    baseConf.verbose = true;
    gulp.task('webpack-server', function() {
        return gulp.src('')
            .pipe(gulpWebpack(baseConf, null))
            .pipe(gulp.dest(path.join(dest)));
    });
};

sharkWebpack.serveStatic = function(conf) {
    var baseConf = addCustomConf(webpackConf('serve', conf.baseConf), conf.webpack);
    // if (conf.dllPlugin) {
    //     generateDlllib('serve', conf);
    //     baseConf.plugins.push(new originalWebpack.DllReferencePlugin(getDllReferencePlugin(conf)));
    // }
    var compile = originalWebpack(baseConf);
    var router = express.Router();
    router.use(webpackDev(compile, {
        headers: { 'Content-Type': 'text/javascript' }
    }));
    router.use(webpackHot(compile));
    return router;
};

function addCustomConf(baseConf, customConf) {
    if (!customConf) return baseConf;
    var targetConf = {};
    if (customConf.addEntry) {
        extend(true, baseConf.entry, customConf.addEntry);
        delete customConf.addEntry;
    } else if (customConf.replaceEntry) {
        baseConf.entry = customConf.replaceEntry;
        delete customConf.replaceEntry;
    }
    extend(true, targetConf, baseConf, customConf);
    return targetConf;
}

function generateDlllib(key, conf) {
    var _dllPlugin = conf.dllPlugin;
    var baseConf = conf.baseConf;
    var name = _dllPlugin.name || 'dlllib';
    var filePath = path.join(baseConf.rootPath, baseConf.tmpDir, 'step1/dll', name + '.js');
    var dest = path.join(baseConf.rootPath, baseConf.tmpDir, 'step1/dll');
    if (fs.existsSync(filePath)) return;
    var _dllConf = dllConf(key, conf);
    return gulp.src('')
        .pipe(gulpWebpack(_dllConf, null))
        .pipe(gulp.dest(path.join(dest)));
}

function getDllReferencePlugin(conf) {
    var baseConf = conf.baseConf;
    var dllPlugin = conf.dllPlugin;
    var name = dllPlugin.name || 'dlllib';
    return {
        context: __dirname,
        manifest: require(path.join(baseConf.rootPath, name + '-manifest.json')),
        name: name,
        sourceType: 'var'
    };
}

module.exports = sharkWebpack;