var util = require('../shark-util');
var gulp = require('gulp');
var clean = require('gulp-clean');
var path = require('path');
var md5 = require('md5');
var zip = require('gulp-zip');
var os = require('os');
var fs = require('fs');

function registerZipTask(conf) {
    if (!conf.zip) return;
    var zipConf = conf.zip;
    var baseConf = conf.baseConf;
    var zipTemps = [];
    var tempDirs = [];
    var md5map = {};
    var zipSteps = [];
    var step = 0;
    registerZipStep();

    function registerZipStep() {
        for (var zipName in zipConf) {
            var dirs = [];
            var taskName = 'zip-stpe-' + (step++);
            zipSteps.push(taskName);
            zipConf[zipName].forEach(function(item) {
                var absPath = path.join(baseConf.rootPath, item);
                var filePath = path.join(absPath, '**');
                dirs.push(filePath);
                tempDirs.push(absPath);
            });
            //生成临时压缩包
            var zipTempName = zipName + '-temp.zip';
            zipTemps.push(zipTempName);
            registerSingleTask(taskName, zipTempName, dirs.join(','));
        }
    }

    function registerSingleTask(taskName, zipName, dirs) {
        gulp.task(taskName, function() {
            return gulp.src(dirs, { base: baseConf.rootPath })
                .pipe(zip(path.join(zipName)))
                .pipe(gulp.dest(path.join(baseConf.rootPath)));
        });
    }

    gulp.task('zip', zipSteps, function() {

        //更改文件的修改时间为统一时间
        for (var zipName in zipConf) {
            zipConf[zipName].forEach(function(item) {
                var absPath = path.join(baseConf.rootPath, item);
                util.changeFileTime(absPath);
            });
        }

        //生成最终压缩包
        var zipPath = path.join(baseConf.rootPath, baseConf.build, 'zip');
        var iosPath = path.join(zipPath, 'ios');
        var androidPath = path.join(zipPath, 'android');
        fs.mkdirSync(zipPath);
        fs.mkdirSync(iosPath);
        fs.mkdirSync(androidPath);

        var fileUrl = '';
        zipTemps.forEach(function(tempZip) {
            var buf = fs.readFileSync(path.join(baseConf.rootPath, tempZip));
            var md5Name = md5(buf);
            fileUrl = os.platform() === 'win32' ? baseConf.md5FileWin : baseConf.md5FileOther;
            if (!fs.existsSync(fileUrl)) {
                fs.writeFileSync(fileUrl, JSON.stringify(md5map), { encoding: 'utf8' });
            }
            var f = fs.readFileSync(fileUrl, 'utf-8');
            md5map = JSON.parse(f);
            var zipName = '',
                zipTime = '';
            if (md5map[md5Name]) {
                zipTime = md5map[md5Name];
                zipName = tempZip.replace('-temp.zip', '') + '-' + md5Name + '-' + zipTime + '.zip';
            } else {
                zipTime = util.formatDate(new Date(), 'yyyyMMddHHmm');
                md5map[md5Name] = zipTime;
                zipName = tempZip.replace('-temp.zip', '') + '-' + md5Name + '-' + zipTime + '.zip';
            }
            fs.writeFileSync(path.join(iosPath, zipName), buf);
            fs.writeFileSync(path.join(androidPath, zipName), buf);
            fs.unlinkSync(path.join(baseConf.rootPath, tempZip));
        });

        //重新写入md5map文件
        fs.writeFileSync(fileUrl, JSON.stringify(md5map), { encoding: 'utf8' });
        //清理临时目录
        return gulp.src(tempDirs, { read: false }).pipe(clean());
    });

}



module.exports = registerZipTask;